/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.minfo;

import org.apache.log4j.Logger;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.minfo.BedDifferencesResult;
import org.dive4elements.river.exports.process.BedDiffHeightYearProcessor;
import org.dive4elements.river.exports.process.KMIndexProcessor;
import org.dive4elements.river.exports.process.Processor;
import org.dive4elements.river.jfree.RiverAnnotation;
import org.dive4elements.river.themes.ThemeDocument;


public class BedDiffHeightYearGenerator
extends BedDiffBaseGenerator
implements FacetTypes
{
    public enum YAXIS {
        D(0), dW(1);

        protected int idx;

        private YAXIS(int c) {
            idx = c;
        }
    }

    /** The logger that is used in this generator. */
    private static Logger logger = Logger.getLogger(BedDiffHeightYearGenerator.class);

    public static final String I18N_CHART_TITLE = "chart.beddifference.height.title";
    public static final String I18N_XAXIS_LABEL = "chart.beddifference.height.xaxis.label";
    public static final String I18N_YAXIS_LABEL = "chart.beddifference.height.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT = "Sohlenhöhen Differenz";
    public static final String I18N_XAXIS_LABEL_DEFAULT = "Fluss-Km";
    public static final String I18N_YAXIS_LABEL_DEFAULT = "delta S [cm / Jahr]";
    public static final String I18N_DW_YAXIS_LABEL_DEFAULT  =
            "delta W [cm]";
    public static final String I18N_DW_YAXIS_LABEL =
            "chart.fixings.longitudinalsection.yaxis.label";

    @Override
    protected YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {

            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
            public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }

    @Override
    public void doOut(ArtifactAndFacet bundle, ThemeDocument attr, boolean visible) {
        String name = bundle.getFacetName();

        logger.debug("doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        Facet facet = bundle.getFacet();

        if (facet == null) {
            return;
        }

        if (bundle.getData(context) instanceof BedDifferencesResult) {
            setContextBounds(bundle);
        }

        Processor processor = new KMIndexProcessor();
        Processor bdyProcessor = new BedDiffHeightYearProcessor();
        if (name.equals(LONGITUDINAL_ANNOTATION)) {
            doAnnotations(
                (RiverAnnotation) bundle.getData(context),
                 bundle,
                 attr,
                 visible);
        }
        else if (processor.canHandle(name)) {
            processor.doOut(this, bundle, attr, visible, YAXIS.dW.idx);
        }
        else if (bdyProcessor.canHandle(name)) {
            bdyProcessor.doOut(this, bundle, attr, visible, YAXIS.D.idx);
        }
        else {
            logger.warn("Unknown facet name " + name);
        }
    }


    @Override
    protected String getDefaultChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }

    @Override
    protected String getDefaultXAxisLabel() {
        return msg(I18N_XAXIS_LABEL,
            I18N_XAXIS_LABEL_DEFAULT,
            new Object[] { getRiverName() });
    }

    @Override
    protected String getDefaultYAxisLabel(int pos) {
        if (pos == YAXIS.D.idx) {
            return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
        }
        else if (pos == YAXIS.dW.idx) {
            return msg(I18N_DW_YAXIS_LABEL, I18N_DW_YAXIS_LABEL_DEFAULT);
        }
        return "default";
    }
}
